/* L-Dic(Linux Dictionary)
 * Copyright (C) 2002 Baek SeungSoon.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>

#include "ldic.h"
#include "rcfile.h"

static char
*homedir(void)
{
	char *home = getenv("HOME");

	if (home != NULL) {
		return home;
	} else {
		struct passwd *pw = getpwuid(getuid());
		if (pw != NULL) {
			return pw->pw_dir;
		} else {
			return NULL;
		}
	}
}

static int
isdir(const char *s)
{
	struct stat st;
	
	if ((!s) || (!*s))
		return 0;
	if (stat(s, &st) < 0)
		return 0;
	if (S_ISDIR(st.st_mode))
		return 1;
	return 0;
}

static int
isfile(const char *s)
{
	struct stat st;
	
	if ((!s) || (!*s))
		return 0;
	if (stat(s, &st) < 0)
		return 0;
	if (S_ISREG(st.st_mode))
		return 1;
	return 0;
}

static void
make_rcfile(void)
{
	FILE *fp;
	char buf[BUFSIZ+1];

	sprintf(buf, "%s/%s/%s", homedir(), LDIC_RC_DIR, RC_FILE_NAME);

	fp = fopen(buf, "w");
	if (fp == NULL) {
		printf("error saving config file: %s\n", buf);
		free(buf);
		return;
	}

	fprintf(fp, "####################################\n");
	fprintf(fp, "# LDic config file version %7s #\n", VERSION);
	fprintf(fp, "####################################\n");
	fprintf(fp, "\n");

	fprintf(fp, "# Dictionary file list #\n");
	fprintf(fp, "dic0=/usr/share/ldic/dic/ldic.dic\n");
	fprintf(fp, "dic1=\n");
	fprintf(fp, "dic2=\n");
	fprintf(fp, "dic3=\n");
	fprintf(fp, "dic4=\n");
	fprintf(fp, "\n");

	fprintf(fp, "###########################\n");
	fprintf(fp, "# End of LDic config file #\n");
	fprintf(fp, "###########################\n");

	fclose(fp);
}

static void
load_option(void)
{
	FILE *fp;
	char buf[1024];
	char *option;
	char *value;

	sprintf(buf, "%s/%s/%s", homedir(), LDIC_RC_DIR, RC_FILE_NAME);

	fp = fopen(buf, "r");
	if (fp == NULL) {
		return;
	}

	while (fgets(buf, 1024, fp)) {
		char *s;
		if (buf[0] < 'a' && buf[0] > 'z')
			continue;

		buf[strlen(buf)-1] = '\0';
		s = strchr(buf, '=');
		if (s == NULL)
			continue;
		
		if (*(s+1) == '\0')
			continue;

		value = strdup(s+1);
		
		*s = '\0';
		option = strdup(buf);

		if (value[0] == '~') {
			sprintf(buf, "%s%s", homedir(), value+1);
			free(value);
			value = strdup(buf);
		}
		
		if (strcmp("dic0", option) == '\0')
			dic_list[0] = strdup(value);
		if (strcmp("dic1", option) == '\0')
			dic_list[1] = strdup(value);
		if (strcmp("dic2", option) == '\0')
			dic_list[2] = strdup(value);
		if (strcmp("dic3", option) == '\0')
			dic_list[3] = strdup(value);
		if (strcmp("dic4", option) == '\0')
			dic_list[4] = strdup(value);

		free(option);
		free(value);
	}
}

extern void
rcfile_load(void)
{
	char buf[BUFSIZ+1];

	sprintf(buf, "%s/%s", homedir(), LDIC_RC_DIR);
	if (!isdir(buf)) {
		mkdir(buf, 0755);
		strcat(buf, "/dic");
		mkdir(buf, 0755);
	}
	
	sprintf(buf, "%s/%s/%s", homedir(), LDIC_RC_DIR, RC_FILE_NAME);

	if (isfile(buf)) {
		load_option();
	} else {
		make_rcfile();
		load_option();
	}
}
